from flask import Flask, request, jsonify, render_template, url_for
from flask_socketio import SocketIO
import requests
import os
import yaml
import logging
import datetime
from dotenv import load_dotenv
import time
import webbrowser
import threading
import matplotlib.pyplot as plt
from visualizer import plot_performance

# Importing custom modules
from data_fetcher import fetch_stock_data
from strategy import generate_signals, train_model, load_model, generate_signals_with_model
from backtester import execute_backtest, analyze_performance
from notifier import send_email
from utils import fetch_balance

# Load environment variables
load_dotenv()

# Define the Flask app
app = Flask(__name__, static_folder='static', template_folder='templates')
app.config['SECRET_KEY'] = os.getenv('SECRET_KEY', 'default_secret_key')
socketio = SocketIO(app)

# Security settings
app.config['SESSION_COOKIE_SECURE'] = True
app.config['REMEMBER_COOKIE_SECURE'] = True
app.config['SESSION_COOKIE_HTTPONLY'] = True
app.config['REMEMBER_COOKIE_HTTPONLY'] = True
app.config['SESSION_PROTECTION'] = 'strong'

# Load configuration from YAML file
with open('config.yaml', 'r', encoding='utf-8') as file:
    config = yaml.safe_load(file)

# Token caching
access_token = None
token_expiry = None

def get_access_token():
    global access_token, token_expiry
    if access_token and token_expiry and token_expiry > datetime.datetime.now():
        return access_token
    url = f"{config['api_config']['access_token']['domain']}{config['api_config']['access_token']['url']}"
    headers = {'content-type': 'application/json; charset=utf-8'}
    grant_type = os.getenv('GRANT_TYPE') or config['api_config']['access_token']['grant_type']
    data = {
        'grant_type': grant_type,
        'client_id': os.getenv('CLIENT_ID') or config['api_config']['access_token']['client_id'],
        'client_secret': os.getenv('CLIENT_SECRET') or config['api_config']['access_token']['client_secret'],
        'scope': os.getenv('SCOPE') or config['api_config']['access_token']['scope']
    }
    response = requests.post(url, headers=headers, json=data)
    response.raise_for_status()
    token_data = response.json()
    access_token = token_data['access_token']
    expires_in = token_data['expires_in']
    token_expiry = datetime.datetime.now() + datetime.timedelta(seconds=expires_in)
    return access_token

@app.route('/')
def index():
    data = fetch_stock_data()
    return render_template('index.html', data=data)

def fetch_portfolio_data():
    access_token = get_access_token()
    headers = {'Authorization': f'Bearer {access_token}', 'Content-Type': 'application/json'}
    response = requests.get(f'{config["api_config"]["portfolio"]["url"]}', headers=headers)
    response.raise_for_status()
    return response.json()

def background_thread():
    while True:
        data = fetch_stock_data()
        portfolio_data = fetch_portfolio_data()
        socketio.emit('updateData', {'stockData': data, 'portfolioData': portfolio_data})
        time.sleep(10)  # Fetch data every 10 seconds

@socketio.on('connect')
def test_connect():
    socketio.start_background_task(target=background_thread)

@socketio.on('startTrading')
def handle_start_trading():
    print('Starting trading...')

@socketio.on('stopTrading')
def handle_stop_trading():
    print('Stopping trading...')

@socketio.on('updateRiskLevel')
def handle_update_risk_level(data):
    print(f'Risk level updated to: {data["riskLevel"]}')

@socketio.on('updateInvestmentAmount')
def handle_update_investment_amount(data):
    print(f'Investment amount updated to: {data["investmentAmount"]}')
    socketio.emit('investmentAmountUpdateSuccess', {'status': 'success', 'investmentAmount': data["investmentAmount"]})

# Function to open the web browser
def open_browser():
    webbrowser.open_new('http://127.0.0.1:5000/')

def plot_data():
    # Load or generate your data here
    portfolio = ...  # Your portfolio DataFrame
    signals = ...  # Your signals DataFrame
    plot_performance(portfolio, signals)
    plt.show()

@app.route('/plot')
def plot_route():
    threading.Thread(target=plot_data).start()
    return "Plotting in background. Check your plots."

if __name__ == '__main__':
    open_browser()
    socketio.run(app, debug=True)
