import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

def sma_crossover_strategy(data, short_window, long_window):
    data['SMA_short'] = data['Close'].rolling(window=short_window, min_periods=1).mean()
    data['SMA_long'] = data['Close'].rolling(window=long_window, min_periods=1).mean()
    data['Signal'] = 0
    data.loc[data.index[short_window:], 'Signal'] = np.where(data['SMA_short'][short_window:] > data['SMA_long'][short_window:], 1, 0)
    return data['Signal']

def ema_crossover_strategy(data, short_window, long_window):
    data['EMA_short'] = data['Close'].ewm(span=short_window, adjust=False).mean()
    data['EMA_long'] = data['Close'].ewm(span=long_window, adjust=False).mean()
    data['Signal'] = 0
    data.loc[data.index[short_window:], 'Signal'] = np.where(data['EMA_short'][short_window:] > data['EMA_long'][short_window:], 1, 0)
    return data['Signal']

def rsi_strategy(data, rsi_period, low_threshold, high_threshold):
    delta = data['Close'].diff()
    gain = (delta.where(delta > 0, 0)).rolling(window=rsi_period).mean()
    loss = (-delta.where(delta < 0, 0)).rolling(window=rsi_period).mean()
    rs = gain / loss
    data['RSI'] = 100 - (100 / (1 + rs))
    data['Signal'] = 0
    data['Signal'] = np.where(data['RSI'] > high_threshold, -1, np.where(data['RSI'] < low_threshold, 1, 0))
    return data['Signal']

def combine_signals(sma_signals, ema_signals, rsi_signals):
    combined = sma_signals + ema_signals + rsi_signals
    combined = np.where(combined > 1, 1, np.where(combined < -1, -1, 0))
    return combined

def test_fetch_historical_data():
    # 예시 데이터 생성 (실제로는 API 등을 통해 데이터를 가져와야 함)
    dates = pd.date_range(start='2020-01-01', periods=100, freq='D')
    prices = np.random.randn(100).cumsum() + 100
    data = pd.DataFrame(data={'Close': prices}, index=dates)
    return data

def mock_execute_backtest(signals):
    returns = np.random.randn(len(signals))  # Mock returns based on signals
    portfolio = pd.DataFrame(data={'Returns': returns.cumsum()})
    return portfolio

def plot_performance(title, portfolio):
    plt.figure(figsize=(12, 6))
    plt.plot(portfolio['Returns'], label='Portfolio Value')
    plt.title(title)
    plt.xlabel('Date')
    plt.ylabel('Cumulative Returns')
    plt.legend()
    plt.show()

# 최적화된 전략 매개변수
sma_params = (3, 20)
ema_params = (3, 20)
rsi_params = (10, 20, 60)

# 최적화된 매개변수를 사용한 신호 생성 함수
def generate_optimized_signals(historical_data, sma_params, ema_params, rsi_params):
    sma_signals = sma_crossover_strategy(historical_data, *sma_params)
    ema_signals = ema_crossover_strategy(historical_data, *ema_params)
    rsi_signals = rsi_strategy(historical_data, *rsi_params)
    combined_signals = combine_signals(sma_signals, ema_signals, rsi_signals)
    return combined_signals

# 역사적 데이터 가져오기
historical_data = test_fetch_historical_data()

# 최적화된 매개변수를 사용하여 신호 생성
optimized_signals = generate_optimized_signals(historical_data, sma_params, ema_params, rsi_params)

# 최적화된 신호로 백테스트 실행
optimized_backtest = mock_execute_backtest(optimized_signals)

# 최적화된 전략 성과 시각화
plot_performance('Optimized Combined Strategy', optimized_backtest)
