import gym
from gym import spaces
import numpy as np
import pandas as pd

class StockTradingEnv(gym.Env):
    metadata = {'render.modes': ['human']}

    def __init__(self, df):
        super(StockTradingEnv, self).__init__()

        self.df = df
        self.reward_range = (0, np.inf)

        # Actions: 0: Buy, 1: Sell, 2: Hold, 3: Buy 50%, 4: Sell 50%
        self.action_space = spaces.Discrete(5)

        # Prices contains the OHCL values and other relevant information
        self.observation_space = spaces.Box(low=0, high=1, shape=(len(df.columns) + 1,), dtype=np.float16)

    def _next_observation(self):
        frame = np.array([
            self.df.loc[self.current_step, 'open'] / self.max_price,
            self.df.loc[self.current_step, 'high'] / self.max_price,
            self.df.loc[self.current_step, 'low'] / self.max_price,
            self.df.loc[self.current_step, 'close'] / self.max_price,
            self.balance / self.initial_balance
        ])
        return frame

    def _take_action(self, action):
        current_price = self.df.loc[self.current_step, "close"]
        if action == 0:  # Buy 100%
            self.balance -= current_price
            self.shares += 1
        elif action == 1:  # Sell 100%
            if self.shares > 0:
                self.balance += current_price
                self.shares -= 1
            else:
                print("No shares to sell")
        elif action == 2:  # Hold
            pass
        elif action == 3:  # Buy 50%
            self.balance -= current_price * 0.5
            self.shares += 0.5
        elif action == 4:  # Sell 50%
            if self.shares >= 0.5:
                self.balance += current_price * 0.5
                self.shares -= 0.5
            else:
                print("Not enough shares to sell")

        self.total_value = self.balance + self.shares * current_price

    def step(self, action):
        self._take_action(action)

        self.current_step += 1

        if self.current_step >= len(self.df) - 1:
            self.current_step = 0  # Episode ends

        reward = self.total_value - self.initial_balance
        done = self.balance <= 0

        obs = self._next_observation()
        return obs, reward, done, {}

    def reset(self):
        self.initial_balance = 1000
        self.balance = self.initial_balance
        self.shares = 0
        self.total_value = self.initial_balance
        self.current_step = 0
        self.max_price = self.df[['open', 'high', 'low', 'close']].max().max()

        return self._next_observation()

    def render(self, mode='human', close=False):
        profit = self.total_value - self.initial_balance
        print(f'Step: {self.current_step}, Balance: {self.balance}, Shares: {self.shares}, Total Value: {self.total_value}, Profit: {profit}')
