import pandas as pd
import numpy as np
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
import joblib
import os
import logging
import matplotlib.pyplot as plt

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

MODEL_FILE = 'model.pkl'

def generate_moving_averages(data, short_window=40, long_window=100):
    data['short_mavg'] = data['price'].rolling(window=short_window, min_periods=1).mean()
    data['long_mavg'] = data['price'].rolling(window=long_window, min_periods=1).mean()
    return data

def generate_signals(data):
    if 'price' not in data.columns:
        raise ValueError("Data must contain a 'price' column")

    signals = data.copy()
    signals['signal'] = 0  # Default signal

    # Calculate moving averages
    signals = generate_moving_averages(signals)

    # Simple strategy: buy when price > short moving average, sell when price < long moving average
    signals.loc[signals['short_mavg'] > signals['long_mavg'], 'signal'] = 1
    signals.loc[signals['short_mavg'] < signals['long_mavg'], 'signal'] = -1

    return signals

def train_model(data, model_file=MODEL_FILE):
    required_columns = ['price', 'short_mavg', 'long_mavg', 'signal']
    if not all(col in data.columns for col in required_columns):
        raise ValueError(f"Data must contain the following columns: {required_columns}")

    data = data.dropna()
    features = data[['price', 'short_mavg', 'long_mavg']]
    labels = data['signal']

    X_train, X_test, y_train, y_test = train_test_split(features, labels, test_size=0.2, random_state=42)

    model = RandomForestClassifier(n_estimators=100, random_state=42)
    model.fit(X_train, y_train)

    accuracy = model.score(X_test, y_test)
    logging.info(f'Model accuracy: {accuracy:.2f}')

    joblib.dump(model, model_file)
    return model

def load_model(model_file=MODEL_FILE):
    if os.path.exists(model_file):
        return joblib.load(model_file)
    else:
        logging.warning(f'Model file {model_file} not found.')
        return None

def generate_signals_with_model(model, data):
    required_columns = ['price', 'short_mavg', 'long_mavg']
    if not all(col in data.columns for col in required_columns):
        raise ValueError(f"Data must contain the following columns: {required_columns}")

    data = data.dropna()
    features = data[required_columns]
    try:
        data['signal'] = model.predict(features)
    except Exception as e:
        logging.error(f'Error during model prediction: {e}')
        data['signal'] = 0
    return data

def sma_crossover_strategy(data, short_window, long_window):
    data['SMA_short'] = data['price'].rolling(window=short_window, min_periods=1).mean()
    data['SMA_long'] = data['price'].rolling(window=long_window, min_periods=1).mean()
    data['Signal'] = 0
    data['Signal'][short_window:] = np.where(data['SMA_short'][short_window:] > data['SMA_long'][short_window:], 1, 0)
    return data['Signal']

def ema_crossover_strategy(data, short_window, long_window):
    data['EMA_short'] = data['price'].ewm(span=short_window, adjust=False).mean()
    data['EMA_long'] = data['price'].ewm(span=long_window, adjust=False).mean()
    data['Signal'] = 0
    data['Signal'][short_window:] = np.where(data['EMA_short'][short_window:] > data['EMA_long'][short_window:], 1, 0)
    return data['Signal']

def rsi_strategy(data, rsi_period, low_threshold, high_threshold):
    delta = data['price'].diff()
    gain = (delta.where(delta > 0, 0)).rolling(window=rsi_period).mean()
    loss = (-delta.where(delta < 0, 0)).rolling(window=rsi_period).mean()
    rs = gain / loss
    data['RSI'] = 100 - (100 / (1 + rs))
    data['Signal'] = 0
    data['Signal'] = np.where(data['RSI'] > high_threshold, -1, np.where(data['RSI'] < low_threshold, 1, 0))
    return data['Signal']

def combine_signals(sma_signals, ema_signals, rsi_signals):
    combined = sma_signals + ema_signals + rsi_signals
    combined = np.where(combined > 1, 1, np.where(combined < -1, -1, 0))
    return combined

def test_fetch_historical_data():
    dates = pd.date_range(start='2020-01-01', periods=100, freq='D')
    prices = np.random.randn(100).cumsum() + 100
    data = pd.DataFrame(data={'price': prices}, index=dates)
    return data

def mock_execute_backtest(signals):
    returns = np.random.randn(len(signals))  # Mock returns based on signals
    portfolio = pd.DataFrame(data={'Returns': returns.cumsum()})
    return portfolio

def plot_performance(title, portfolio):
    plt.figure(figsize=(12, 6))
    plt.plot(portfolio['Returns'], label='Portfolio Value')
    plt.title(title)
    plt.xlabel('Date')
    plt.ylabel('Cumulative Returns')
    plt.legend()
    plt.show()

# Optimized strategy parameters
sma_params = (3, 20)
ema_params = (3, 20)
rsi_params = (10, 20, 60)

def generate_optimized_signals(historical_data, sma_params, ema_params, rsi_params):
    sma_signals = sma_crossover_strategy(historical_data, *sma_params)
    ema_signals = ema_crossover_strategy(historical_data, *ema_params)
    rsi_signals = rsi_strategy(historical_data, *rsi_params)
    combined_signals = combine_signals(sma_signals, ema_signals, rsi_signals)
    return combined_signals

if __name__ == "__main__":
    historical_data = test_fetch_historical_data()
    
    # Generate initial signals
    signals = generate_signals(historical_data)

    # Train and save the model
    model = train_model(signals)

    # Load the model and generate signals with it
    model = load_model()
    if model:
        signals_with_model = generate_signals_with_model(model, signals)

    # Generate optimized signals using technical indicators
    optimized_signals = generate_optimized_signals(historical_data, sma_params, ema_params, rsi_params)

    # Execute backtest with optimized signals
    optimized_backtest = mock_execute_backtest(optimized_signals)

    # Plot performance of the optimized strategy
    plot_performance('Optimized Combined Strategy', optimized_backtest)
