import tkinter as tk
from tkinter import messagebox
import asyncio
import aiohttp
import yaml
import logging
from app import get_access_token  # Import the get_access_token function from app.py

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

# Load configuration
def load_config():
    with open('config.yaml', 'r', encoding='utf-8') as file:
        config = yaml.safe_load(file)
    return config

# Get API details
def get_api_details(config):
    return config['trading_bot']['api']

# Fetch account information
async def fetch_account_info(api_details):
    base_url = api_details['base_url']
    access_token = get_access_token()  # Get the access token

    headers = {
        'Authorization': f'Bearer {access_token}',
        'appKey': api_details['app_key'],
        'appSecret': api_details['app_secret']
    }

    async with aiohttp.ClientSession() as session:
        try:
            async with session.get(f"{base_url}/uapi/domestic-stock/v1/trading/inquire-balance", headers=headers) as response:
                response.raise_for_status()
                return await response.json()
        except aiohttp.ClientError as e:
            logging.error(f"Error fetching account info: {e}")
            return None

# Update account information
async def update_account_info(api_details, label):
    while True:
        data = await fetch_account_info(api_details)
        if data:
            try:
                output = data['output'][0]
                info = (
                    f"총 손익: {output['evlu_pfls_amt']}\n"
                    f"총 매입: {output['pchs_amt_smtl']}\n"
                    f"총 평가: {output['evlu_amt_smtl']}\n"
                    f"실현손익: {output['real_evlu_pfls']}\n"
                    f"추정자산: {output['nxtr_prvs_rcdl_pfls']}"
                )
                label.config(text=info)
            except (KeyError, IndexError) as e:
                logging.error(f"Error parsing account info: {e}")
        await asyncio.sleep(5)  # Update every 5 seconds

# Fetch real-time data
async def fetch_real_time_data(api_details):
    base_url = api_details['base_url']
    access_token = get_access_token()  # Get the access token

    headers = {
        'Authorization': f'Bearer {access_token}',
        'appKey': api_details['app_key'],
        'appSecret': api_details['app_secret']
    }

    async with aiohttp.ClientSession() as session:
        try:
            async with session.get(f"{base_url}/uapi/domestic-stock/v1/quotations/inquire-price", headers=headers) as response:
                response.raise_for_status()
                return await response.json()
        except aiohttp.ClientError as e:
            logging.error(f"Error fetching real-time data: {e}")
            return None

# Analyze data
async def analyze_data(data):
    logging.info("Analyzing data...")
    return {"signal": "buy", "symbol": "AAPL", "quantity": 10}

# Full sell
async def full_sell(api_details):
    base_url = api_details['base_url']
    access_token = get_access_token()  # Get the access token

    headers = {
        'Authorization': f'Bearer {access_token}',
        'appKey': api_details['app_key'],
        'appSecret': api_details['app_secret']
    }

    async with aiohttp.ClientSession() as session:
        try:
            async with session.get(f"{base_url}/uapi/domestic-stock/v1/trading/inquire-balance", headers=headers) as response:
                response.raise_for_status()
                holdings = await response.json()
        except aiohttp.ClientError as e:
            logging.error(f"Error fetching holdings: {e}")
            return

        tasks = []
        for holding in holdings.get('output', []):
            symbol = holding['pdno']
            quantity = holding['hldg_qty']

            order_payload = {
                'CANO': api_details['account_number'],
                'ACNT_PRDT_CD': '01',
                'PDNO': symbol,
                'ORD_DVSN': '00',
                'ORD_QTY': quantity,
                'ORD_UNPR': '0'
            }

            tasks.append(place_order(session, base_url, headers, order_payload))

        await asyncio.gather(*tasks)

async def place_order(session, base_url, headers, order_payload):
    try:
        async with session.post(f"{base_url}/uapi/domestic-stock/v1/trading/order-cash", json=order_payload, headers=headers) as response:
            response.raise_for_status()
            logging.info("Order placed successfully")
    except aiohttp.ClientError as e:
        logging.error(f"Order placement failed: {e}")

def run_asyncio_task(task):
    asyncio.run(task)

def main():
    config = load_config()
    api_details = get_api_details(config)
    root = tk.Tk()
    root.title("Stock Trading Bot")
    label = tk.Label(root, text="Fetching account info...")
    label.pack()
    
    # Start asyncio task in Tkinter
    asyncio.create_task(update_account_info(api_details, label))
    
    root.mainloop()

if __name__ == "__main__":
    main()
